/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package p4uclient;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

/**
 * アプリ全体を管理
 * @author tondol
 */
public class MainApp {
    static Option option = null;
    static ClientSession client = null;
    static MainView window = null;
    static Vector<FileItem> sharedItems = null;
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        window = MainView.getInstance();
        option = new Option();
        sharedItems = new Vector<FileItem>();
    }

    /**
     * 終了時の処理
     */
    public static void exit() {
        if (client != null) close();
        if (option != null) option.save();
        System.exit(0);
    }
    
    /**
     * 各セッションを開始
     */
    public static void run() {
        try {
            window.setState(MainView.BUSY);
            client = new ClientSession();
            updateItems();
            window.setState(MainView.CONNECTED);
            window.putMessage("接続完了 <" + option.host + ":" + option.port + ">");
        } catch(IOException e) {
            window.setState(MainView.CLOSED);
            window.putMessage("接続失敗 <" + option.host + ":" + option.port + ">");
        }
    }
    
    /**
     * 各セッションを終了
     */
    public static void close() {
        try {
            window.setState(MainView.BUSY);
            clearItems();
            client.close();
            client = null;
        } catch(IOException e) {
            ;
        } finally {
            window.setState(MainView.CLOSED);
            window.putMessage("接続切断");
        }
    }
    
    /**
     * サーバーの共有リストを検索
     * @param word 検索ワード
     */
    public static void searchItems(String word) {
        try {
            // 検索要求を送信
            Vector<FileItem> items;
            if (word.length() == 0) word = ".";
            client.sendMessage("LS " + word);
            // 検索結果を取得
            int size = Integer.parseInt(client.recvMessage());
            items = new Vector<FileItem>(size);
            for(int i=0;i<size;i++){
                String str = client.recvMessage();
                items.add(new FileItem(str));
            }
            // 表に反映する
            window.setTableSearch(items);
            window.putMessage("検索完了 <" + word + ">");
        } catch(IOException e) {
            window.putMessage("検索失敗 <" + word + ">");
            close();
        }
    }
    
    /**
     * 共有リストを更新
     */
    public static void updateItems() {
        try {
            window.setState(MainView.BUSY);
            // ファイルリスト取得
            File array[] = new File(option.upDir).listFiles();
            Vector<FileItem> recent = new Vector<FileItem>(array.length);
            for(int i=0;i<array.length;i++){
                if (!array[i].isFile()) continue;
                recent.add(new FileItem(array[i]));
            }
            // 現在のリストを複製
            Vector<FileItem> backup = new Vector<FileItem>(sharedItems.size());
            for(int i=0;i<sharedItems.size();i++){
                backup.add(sharedItems.get(i));
            }
            // 新規取得分を追加
            for(int i=0;i<recent.size();i++){
                FileItem item = recent.get(i);
                if (backup.contains(item)) continue;
                sharedItems.add(item);
                client.addItem(item);
            }
            // 新規消去分を削除
            for(int i=0;i<backup.size();i++){
                FileItem item = backup.get(i);
                if (recent.contains(item)) continue;
                sharedItems.remove(item);
                client.delItem(item);
            }
            window.putMessage("スキャン完了 <" + option.upDir + ">");
        } catch(IOException e) {
            window.putMessage("スキャン失敗 <" + option.upDir + ">");
        } catch(NullPointerException e) {
            window.putMessage("無効な送信フォルダ <" + option.upDir + ">");
        } finally {
            window.setState(MainView.CONNECTED);
        }
    }
    
    /**
     * 共有リストをクリア
     * @throws java.io.IOException
     */
    public static void clearItems() throws IOException {
        for(int i=0;i<sharedItems.size();i++){
            client.sendMessage("DEL " + sharedItems.get(i));
            window.putMessage(client.recvMessage());
        }
        sharedItems.clear();
    }
    
    /**
     * ファイルを受信
     * @param hash
     */
    public static void runTask(String hash) {
        // 受信タスクを追加
    }
    
    /**
     * タスクをキャンセル
     */
    public static void killTask() {
        // 指定タスクを中断
    }
}
