/*
 * MainView.java
 *
 * Created on 2008/05/29, 18:01
 */

package p4uclient;

import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;

/**
 * アプリのインターフェイスを管理
 * @author  tondol
 */
public class MainView extends javax.swing.JFrame {
    private static MainView instance = null;
    FileItemTableModel modelSearch = null;
    TaskTableModel modelTasks = null;
    // 状態設定用の定数
    final static int CLOSED = 0;
    final static int CONNECTED = 1;
    final static int BUSY = 2;
    
    /** Creates new form MainView */
    private MainView() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            SwingUtilities.updateComponentTreeUI(this);
        } catch(Exception e) {
            ;
        } finally {
            modelSearch = new FileItemTableModel();
            modelTasks = new TaskTableModel();
            initComponents();
            setState(CLOSED);
        }
    }
    
    /**
     * インスタンスを生成しつつ表示
     */
    public static MainView getInstance() {
        if (instance == null) {
            instance = new MainView();
            java.awt.EventQueue.invokeLater(new Runnable() {
                public void run() {
                    instance.setVisible(true);
                }
            });
        }
        return instance;
    }
    
    /**
     * インターフェイスの状態を変更
     * @param state
     */
    @Override
    public void setState(int state) {
        switch(state) {
            case CLOSED:
                btnRun.setEnabled(true);
                btnUpdate.setEnabled(false);
                btnClose.setEnabled(false);
                btnConfig.setEnabled(true);
                txtSearch.setEnabled(false);
                btnSearch.setEnabled(false);
                btnDownload.setEnabled(false);
                btnKill.setEnabled(false);
                txtSearch.setText("");
                modelSearch.clearItems();
                modelTasks.clearItems();
                break;
            case CONNECTED:
                btnRun.setEnabled(false);
                btnUpdate.setEnabled(true);
                btnClose.setEnabled(true);
                btnConfig.setEnabled(true);
                txtSearch.setEnabled(true);
                btnSearch.setEnabled(true);
                btnDownload.setEnabled(true);
                btnKill.setEnabled(false);
                break;
            case BUSY:
                btnRun.setEnabled(false);
                btnUpdate.setEnabled(false);
                btnClose.setEnabled(true);
                btnConfig.setEnabled(false);
                txtSearch.setEnabled(false);
                btnSearch.setEnabled(false);
                btnDownload.setEnabled(false);
                btnKill.setEnabled(false);
                break;
        }
    }
    
    /**
     * 外部から表を変更する
     * @param items
     */
    public void setTableSearch(final Vector<FileItem> items) {
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                modelSearch.setItems(items);
                return null;
        }};
        worker.execute();
    }
    public void setTableTasks(final Vector<TaskSession> items) {
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                modelTasks.setItems(items);
                return null;
        }};
        worker.execute();
    }
    
    /**
     * ログにメッセージを書き出す
     * @param msg
     */
    public void putMessage(String msg) {
        taOutput.append(msg + "\n");
        taOutput.setCaretPosition(taOutput.getText().length());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        tabPane = new javax.swing.JTabbedPane();
        panelSearch = new javax.swing.JPanel();
        scrollSearch = new javax.swing.JScrollPane();
        tableSearch = new javax.swing.JTable();
        txtSearch = new javax.swing.JTextField();
        btnSearch = new javax.swing.JButton();
        btnDownload = new javax.swing.JButton();
        panelTasks = new javax.swing.JPanel();
        scrollTasks = new javax.swing.JScrollPane();
        tableTasks = new javax.swing.JTable();
        btnKill = new javax.swing.JButton();
        panelLog = new javax.swing.JPanel();
        scrollLog = new javax.swing.JScrollPane();
        taOutput = new javax.swing.JTextArea();
        btnClear = new javax.swing.JButton();
        btnRun = new javax.swing.JButton();
        btnClose = new javax.swing.JButton();
        btnConfig = new javax.swing.JButton();
        btnUpdate = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("P4UClient");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        tabPane.setFont(tabPane.getFont().deriveFont(tabPane.getFont().getSize()+1f));
        tabPane.setMinimumSize(new java.awt.Dimension(100, 100));
        tabPane.setPreferredSize(new java.awt.Dimension(200, 150));

        tableSearch.setFont(tableSearch.getFont().deriveFont(tableSearch.getFont().getSize()+2f));
        tableSearch.setModel(this.modelSearch);
        tableSearch.setOpaque(false);
        tableSearch.setRowHeight(18);
        tableSearch.setRowMargin(2);
        tableSearch.getTableHeader().setReorderingAllowed(false);
        tableSearch.setVerifyInputWhenFocusTarget(false);
        scrollSearch.setViewportView(tableSearch);

        txtSearch.setFont(txtSearch.getFont().deriveFont(txtSearch.getFont().getSize()+2f));
        txtSearch.setMargin(new java.awt.Insets(3, 3, 3, 3));
        txtSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchTextAction(evt);
            }
        });
        txtSearch.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                slectTextAction(evt);
            }
        });

        btnSearch.setText("検索");
        btnSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchTextAction(evt);
            }
        });

        btnDownload.setText("取得");
        btnDownload.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                downloadItemAction(evt);
            }
        });

        javax.swing.GroupLayout panelSearchLayout = new javax.swing.GroupLayout(panelSearch);
        panelSearch.setLayout(panelSearchLayout);
        panelSearchLayout.setHorizontalGroup(
            panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelSearchLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelSearchLayout.createSequentialGroup()
                        .addComponent(txtSearch, javax.swing.GroupLayout.DEFAULT_SIZE, 374, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnSearch)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnDownload))
                    .addComponent(scrollSearch, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 516, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelSearchLayout.setVerticalGroup(
            panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelSearchLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelSearchLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(txtSearch, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnSearch)
                    .addComponent(btnDownload))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(scrollSearch, javax.swing.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
                .addContainerGap())
        );

        tabPane.addTab(" 検索 ", panelSearch);

        tableTasks.setFont(tableTasks.getFont().deriveFont(tableTasks.getFont().getSize()+2f));
        tableTasks.setModel(this.modelTasks);
        tableTasks.setOpaque(false);
        tableTasks.setRowHeight(18);
        tableTasks.setRowMargin(2);
        scrollTasks.setViewportView(tableTasks);

        btnKill.setText("中断");
        btnKill.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                killTaskAction(evt);
            }
        });

        javax.swing.GroupLayout panelTasksLayout = new javax.swing.GroupLayout(panelTasks);
        panelTasks.setLayout(panelTasksLayout);
        panelTasksLayout.setHorizontalGroup(
            panelTasksLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelTasksLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelTasksLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(scrollTasks, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 516, Short.MAX_VALUE)
                    .addComponent(btnKill))
                .addContainerGap())
        );
        panelTasksLayout.setVerticalGroup(
            panelTasksLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelTasksLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(scrollTasks, javax.swing.GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnKill)
                .addContainerGap())
        );

        tabPane.addTab(" タスク ", panelTasks);

        taOutput.setColumns(20);
        taOutput.setEditable(false);
        taOutput.setRows(5);
        scrollLog.setViewportView(taOutput);

        btnClear.setText("クリア");
        btnClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearLogMessages(evt);
            }
        });

        javax.swing.GroupLayout panelLogLayout = new javax.swing.GroupLayout(panelLog);
        panelLog.setLayout(panelLogLayout);
        panelLogLayout.setHorizontalGroup(
            panelLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelLogLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(scrollLog, javax.swing.GroupLayout.DEFAULT_SIZE, 516, Short.MAX_VALUE)
                    .addComponent(btnClear))
                .addContainerGap())
        );
        panelLogLayout.setVerticalGroup(
            panelLogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelLogLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(scrollLog, javax.swing.GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnClear)
                .addContainerGap())
        );

        tabPane.addTab(" ログ ", panelLog);

        btnRun.setText("接続");
        btnRun.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });

        btnClose.setText("切断");
        btnClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });

        btnConfig.setText("設定");
        btnConfig.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                configAction(evt);
            }
        });

        btnUpdate.setText("スキャン");
        btnUpdate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(tabPane, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 549, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(btnRun)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnClose)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnUpdate)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 253, Short.MAX_VALUE)
                        .addComponent(btnConfig)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnRun)
                    .addComponent(btnConfig)
                    .addComponent(btnClose)
                    .addComponent(btnUpdate))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(tabPane, javax.swing.GroupLayout.DEFAULT_SIZE, 324, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        java.awt.Dimension dialogSize = getSize();
        setLocation((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
    }// </editor-fold>//GEN-END:initComponents

    /**
     * 基本的なアクション
     * @param evt
     */
    private void mainActions(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mainActions
        final String action = evt.getActionCommand();
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                if (action.equals("接続")) {
                    MainApp.run();
                } else if (action.equals("スキャン")) {
                    MainApp.updateItems();
                } else if (action.equals("切断")) {
                    MainApp.close();
                }
                return null;
        }};
        worker.execute();
}//GEN-LAST:event_mainActions

    /**
     * テキストボックスを選択
     * @param evt
     */
    private void slectTextAction(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_slectTextAction
        javax.swing.JTextField txtbox = (javax.swing.JTextField)evt.getSource();
        txtbox.setSelectionStart(0);
        txtbox.setSelectionEnd(txtbox.getText().length());
}//GEN-LAST:event_slectTextAction

    /**
     * ファイルを検索する
     * @param evt
     */
    private void searchTextAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchTextAction
        final String word = txtSearch.getText();
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                MainApp.searchItems(word);
                return null;
        }};
        worker.execute();
}//GEN-LAST:event_searchTextAction

    /**
     * 選択したファイルをダウンロード
     * @param evt
     */
    private void downloadItemAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_downloadItemAction
        // 選択されたアイテムを取得 -> FileInfoに変換
        SwingWorker worker = new SwingWorker() {
            @Override protected Object doInBackground() {
                // ここで呼び出す
                return null;
        }};
        worker.execute();
    }//GEN-LAST:event_downloadItemAction

    /**
     * ウィンドウを閉じる
     * @param evt
     */
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        int option = JOptionPane.showConfirmDialog(this, "本当に終了しちゃいますよ？",
                "P4UClient - 確認", JOptionPane.YES_NO_OPTION);
        if (option != JOptionPane.YES_OPTION) return;
        MainApp.exit();
    }//GEN-LAST:event_formWindowClosing

    /**
     * ログをクリア
     * @param evt
     */
    private void clearLogMessages(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearLogMessages
        int option = JOptionPane.showConfirmDialog(this, "ログをクリアしちゃいますよ？",
                "P4UClient - 確認", JOptionPane.YES_NO_OPTION);
        if (option != JOptionPane.YES_OPTION) return;
        taOutput.setText("");
    }//GEN-LAST:event_clearLogMessages

    /**
     * タスクを中断
     * @param evt
     */
    private void killTaskAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_killTaskAction
        int option = JOptionPane.showConfirmDialog(this, "タスクを中断しちゃいますよ？",
                "P4UClient - 確認", JOptionPane.YES_NO_OPTION);
        if (option != JOptionPane.YES_OPTION) return;
        // 選択されたアイテムを取得 -> TaskSessionに変換
        SwingWorker worker = new SwingWorker() {
            @Override
            protected Object doInBackground() {
                // ここで呼び出す
                return null;
        }};
}//GEN-LAST:event_killTaskAction

    /**
     * 設定ダイアログを開く
     * @param evt
     */
    private void configAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_configAction
        OptionBox.showOptionBox();
    }//GEN-LAST:event_configAction

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnClear;
    private javax.swing.JButton btnClose;
    private javax.swing.JButton btnConfig;
    private javax.swing.JButton btnDownload;
    private javax.swing.JButton btnKill;
    private javax.swing.JButton btnRun;
    private javax.swing.JButton btnSearch;
    private javax.swing.JButton btnUpdate;
    private javax.swing.JPanel panelLog;
    private javax.swing.JPanel panelSearch;
    private javax.swing.JPanel panelTasks;
    private javax.swing.JScrollPane scrollLog;
    private javax.swing.JScrollPane scrollSearch;
    private javax.swing.JScrollPane scrollTasks;
    private javax.swing.JTextArea taOutput;
    private javax.swing.JTabbedPane tabPane;
    private javax.swing.JTable tableSearch;
    private javax.swing.JTable tableTasks;
    private javax.swing.JTextField txtSearch;
    // End of variables declaration//GEN-END:variables
}
